;################################################################################
;#										#
;# copyright (c) 2012-2013 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public				#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------
; some macros...
;-------------------------------------------------------------------------------
.macro get_tcnt1
.if ctype == 8
		in	r16,TCNT1L
		in	r16,TCNT1H
.else
		lds	r16,TCNT1L
		lds	r16,TCNT1H
.endif
.endm


.macro set_ocr1a
.if ctype == 8
		out	OCR1AH,XH
		out	OCR1AL,XL
.else
		sts	OCR1AH,XH
		sts	OCR1AL,XL
.endif
.endm

.macro set_ocr1b
.if ctype == 8
		out	OCR1BH,XH
		out	OCR1BL,XL
.else
		sts	OCR1BH,XH
		sts	OCR1BL,XL
.endif
.endm


;-------------------------------------------------------------------------------
; goto main loop if program is stopped
;-------------------------------------------------------------------------------
run_end:	cbi	DDRB,7			;run led off
		sbi	PORTB,7
		rcall	wait_nokey
		rjmp	start_1			;restart

;-------------------------------------------------------------------------------
; set variables and start interpreter
;-------------------------------------------------------------------------------
run:		sts	autostart,const_0
		clr	block			;start with main block
		clr	element			;start with first element
		clr	variable
		sts	aux_var1,variable
		sts	aux_var2,variable
		ldi	first_vert,0xff		;no vertical stack
		;get I/O setting
		ldi	ZL,LOW(prog_1*2+992)	;I/O config address
		ldi	ZH,HIGH(prog_1*2+992)
		ldi	XL,4
		lds	XH,prog_num
		andi	XH,0x03
		mul	XL,XH
		add	ZH,r0
		lpm	ZL,Z
		sts	channel_out,ZL	;set I/O
		sbi	DDRB,7		;run led on
		cbi	PORTB,7
		sts	channel_pwm+0, const_0
		sts	channel_pwm+1, const_0
		sts	channel_pwm+2, const_0
		sts	channel_pwm+3, const_0
		sts	channel_pwm+4, const_0
		sts	channel_pwm+5, const_0
		rcall	clr_segm
		ldi	XL,100
		ldi	XH,0
		rcall	wait1_tick


		;the interpreter loop
run_loop:	sbis	PINB,6			;skip if stop button is not pressed
		rjmp	run_end			;exit the interpreter
		cpi	element,32
		brcs	run_loop_0
		rjmp	run_loop_xt

		;get element data
run_loop_0:	ldi	ZL,LOW(prog_1*2+32)
		ldi	ZH,HIGH(prog_1*2+32)
		ldi	XL,4
		lds	XH,prog_num
		andi	XH,0x03
		mul	XL,XH
		add	ZH,r0			;add prog offset
		ldi	XL,192			;bytes per box
		mul	XL,block
		add	ZL,r0			;add box offset
		adc	ZH,r1
		ldi	XL,6			;bytes per element
		mul	XL,element
		add	ZL,r0			;add element offset
		adc	ZH,r1
		lpm	r12,Z+			;type
		lpm	r13,Z+			;value
		lpm	r14,Z+			;next hor
		lpm	r15,Z+			;next vert
		ldi	ZL,10			;VAL wert korrigieren
		sub	r13,ZL
		mov	ZL,r12
		cpi	ZL,100
		brcc	run_loop_xt
		mov	ZL,r12
		cpi	ZL,8			;connectors cannot connect vertical
		brcc	run_loop_1
		clr	r15			;no vertical connect
		dec	r15
run_loop_1:	ldi	ZL,LOW(run_jtab)
		ldi	ZH,HIGH(run_jtab)
		add	ZL,r12
		adc	ZH,const_0
		clt
		icall				;do element

		brtc	run_loop_false		;branch if result is false

		;result is true
		sbrc	r14,7			;check horizontal for valid
		rjmp	run_loop_h0		;no next horizontal
		mov	element,r14		;next hor element is next element
		mov	first_vert,r14		;this can also be the first element of vertical stack
		ldi	tempreg4,0x01		;horizontal
		rjmp	run_loop		;do next element

		;check for vertical
run_loop_h0:	sbrc	r15,7			;check vertical for valid
		rjmp	run_loop_h1		;no next vertical
		mov	element,r15		;set next vertical element
		rjmp	run_loop		;do next vertical

run_loop_h1:	cp	first_vert,element	;is there more than one element in vertical stack?
		breq	run_loop_xt		;branch if not
		mov	element,first_vert	;repeat vertical stack
		rjmp	run_loop

		;result is false
run_loop_false:	sbrc	r15,7			;vertical successor?
		rjmp	run_loop_h2		;no
		mov	element,r15
		rjmp	run_loop		;do next vertical

run_loop_h2:	mov	element,first_vert	;repeat vertical stack
		rjmp	run_loop

run_loop_xt:	ldi	r16,0x040		;show end (--)
		mov	disp_d1,r16
		mov	disp_d0,r16
run_loop_xt1:	sbic	PINB,6			;wait for stop button
		rjmp	run_loop_xt1
		rjmp	run_end

;-------------------------------------------------------------------------------
; Sprungtabelle für die verschiedenen Blöcke
;-------------------------------------------------------------------------------
run_jtab:	rjmp	element_none		;code 00
		rjmp	element_none		;code 01
		rjmp	element_none		;code 02
		rjmp	element_none		;code 03
		rjmp	element_none		;code 04 sub entry
		rjmp	returnsub		;code 05
		rjmp	element_none		;code 06
		rjmp	element_none		;code 07
		rjmp	element_none		;code 08
		rjmp	element_none		;code 09

		rjmp	wait_fast		;code 10
		rjmp	wait_slow		;code 11
		rjmp	wait_fast_int		;code 12
		rjmp	wait_slow_int		;code 13
		rjmp	element_none		;code 14
		rjmp	element_none		;code 15
		rjmp	element_none		;code 16
		rjmp	element_none		;code 17
		rjmp	element_none		;code 18
		rjmp	element_none		;code 19

		rjmp	element_none		;code 20
		rjmp	element_none		;code 21
		rjmp	element_none		;code 22
		rjmp	element_none		;code 23
		rjmp	element_none		;code 24
		rjmp	element_none		;code 25
		rjmp	element_none		;code 26
		rjmp	element_none		;code 27
		rjmp	element_none		;code 28
		rjmp	element_none		;code 29

		rjmp	set_pwm0		;code 30
		rjmp	var_pwm0		;code 31
		rjmp	set_pwm1		;code 32
		rjmp	var_pwm1		;code 33
		rjmp	set_pwm2		;code 34
		rjmp	var_pwm2		;code 35
		rjmp	set_pwm3		;code 36
		rjmp	var_pwm3		;code 37
		rjmp	set_pwm4		;code 38
		rjmp	var_pwm4		;code 39

		rjmp	set_pwm5		;code 40
		rjmp	var_pwm5		;code 41
		rjmp	element_none		;code 42
		rjmp	element_none		;code 43
		rjmp	set_motor1		;code 44
		rjmp	var_motor1		;code 45
		rjmp	set_motor2		;code 46
		rjmp	var_motor2		;code 47
		rjmp	element_none		;code 48
		rjmp	element_none		;code 49

		rjmp	sound_on		;code 50
		rjmp	sound_off		;code 51
		rjmp	exchange1		;code 52
		rjmp	exchange2		;code 53
		rjmp	add1			;code 54
		rjmp	add2			;code 55
		rjmp	element_none		;code 56
		rjmp	element_none		;code 57
		rjmp	element_none		;code 58
		rjmp	element_none		;code 59

		rjmp	element_none		;code 60
		rjmp	element_none		;code 61
		rjmp	element_none		;code 62
		rjmp	element_none		;code 63
		rjmp	element_none		;code 64
		rjmp	element_none		;code 65
		rjmp	element_none		;code 66
		rjmp	element_none		;code 67
		rjmp	set_cdat		;code 68
		rjmp	send_cdat		;code 69

		rjmp	swi0			;code 70
		rjmp	swi1			;code 71
		rjmp	swi2			;code 72
		rjmp	swi3			;code 73
		rjmp	swi4			;code 74
		rjmp	swi5			;code 75
		rjmp	element_none		;code 76
		rjmp	element_none		;code 77
		rjmp	set_var			;code 78
		rjmp	mod_var			;code 79

		rjmp	inv_var			;code 80
		rjmp	rand_var		;code 81
		rjmp	var_gt			;code 82
		rjmp	var_lt			;code 83
		rjmp	var_eq			;code 84
		rjmp	var_ne			;code 85
		rjmp	mess_0			;code 86
		rjmp	mess_1			;code 87
		rjmp	mess_2			;code 88
		rjmp	mess_3			;code 89

		rjmp	mess_4			;code 90
		rjmp	mess_5			;code 91
		rjmp	view_var		;code 92
		rjmp	hide_var		;code 93
		rjmp	element_none		;code 94
		rjmp	element_none		;code 95
		rjmp	callsub1		;code 96
		rjmp	callsub2		;code 97
		rjmp	callsub3		;code 98
		rjmp	callsub4		;code 99

element_none:	set				;true
		ret				;empty element

callsub1:	sts	sub_ele,element		;store current element
		sts	sub_r14,r14		;next hor element
		sts	sub_var,variable
		ldi	block,1
		ldi	element,0
		clr	variable
		ldi	first_vert,0xff
		set				;is always true
		pop	r0			;kill stack
		pop	r0
		rjmp	run_loop_0

callsub2:	sts	sub_ele,element
		sts	sub_r14,r14		;next hor element
		sts	sub_var,variable
		ldi	block,2
		ldi	element,0
		clr	variable
		ldi	first_vert,0xff
		set
		pop	r0			;kill stack
		pop	r0
		rjmp	run_loop_0

callsub3:	sts	sub_ele,element
		sts	sub_r14,r14		;next hor element
		sts	sub_var,variable
		ldi	block,3
		ldi	element,0
		clr	variable
		ldi	first_vert,0xff
		set
		pop	r0			;kill stack
		pop	r0
		rjmp	run_loop_0

callsub4:	sts	sub_ele,element
		sts	sub_r14,r14		;next hor element
		sts	sub_var,variable
		ldi	block,4
		ldi	element,0
		clr	variable
		ldi	first_vert,0xff
		set
		pop	r0			;kill stack
		pop	r0
		rjmp	run_loop_0

returnsub:	lds	element,sub_ele		;restore element nr
		lds	variable,sub_var	;restore var
		lds	r14,sub_r14
		clr	r15
		dec	r15			;no next vert
		ldi	block,0
		ldi	first_vert,0xff
		set
		ret


;-------------------------------------------------------------------------------
;wait X ticks (10ms)
;-------------------------------------------------------------------------------
wait_tick:	movw	count_l,XL		;store value
wait_tick_1:	movw	XL,count_l
		or	XL,XH
		brne	wait_tick_1
		ret

wait1_tick:	movw	count_l,XL		;store value
wait1_tick_1:	movw	XL,count_l
		sbis	PINB,6
		rjmp	wait1_tick_e
		or	XL,XH
		brne	wait1_tick_1
wait1_tick_e:	ret


wait_fast:	cp	r13,const_0
		breq	wait_fast_1
		ldi	XL,10
		mul	XL,r13
		movw	XL,r0
		rcall	wait1_tick
wait_fast_1:	set
		ret

wait_slow:	cp	r13,const_0
		breq	wait_slow_1
		ldi	XL,100
		mul	XL,r13
		movw	XL,r0
		rcall	wait1_tick
wait_slow_1:	set
		ret

wait_fast_int:	cp	r13,const_0		;ignore time 0
		breq	wait_fast_int2		;0 is always true
		sbrs	tempreg4,0		;skip if horizontal
		rjmp	wait_fast_int1		;vertical

		ldi	XL,10
		mul	XL,r13
		movw	count_l,r0		;store value
		clr	tempreg4
		clt				;set false
		ret

		;vertical check
wait_fast_int1:	movw	XL,count_l
		sbis	PINB,6
		rjmp	wait_fast_int3
		or	XL,XH
		brne	wait_fast_int3
wait_fast_int2:	set				;set true
wait_fast_int3:	ret


wait_slow_int:	cp	r13,const_0		;ignore time 0
		breq	wait_slow_int2		;0 is always true
		sbrs	tempreg4,0		;skip if horizontal
		rjmp	wait_slow_int1		;vertical

		ldi	XL,100
		mul	XL,r13
		movw	count_l,r0		;store value
		clr	tempreg4
		clt				;set false
		ret

		;vertical check
wait_slow_int1:	movw	XL,count_l
		sbis	PINB,6
		rjmp	wait_slow_int3
		or	XL,XH
		brne	wait_slow_int3
wait_slow_int2:	set				;set true
wait_slow_int3:	ret



;-------------------------------------------------------------------------------
;wait X ticks (10ms)
;-------------------------------------------------------------------------------
sound_on:	mov	r16,r13
		cpi	r16,21
		brcc	sound_off
		sts	tone_num,r16
		set
		ret

sound_off:	ldi	r16,0xff
		sts	tone_num,r16
		set
		ret


;-------------------------------------------------------------------------------
;get ADC
;-------------------------------------------------------------------------------
get_adc5:	ldi	XL,0x60
		rjmp	get_adc

get_adc4:	ldi	XL,0x61
		rjmp	get_adc

get_adc3:	ldi	XL,0x62
		rjmp	get_adc

get_adc2:	ldi	XL,0x63
		rjmp	get_adc

get_adc1:	ldi	XL,0x64
		rjmp	get_adc

.if ctype == 8
get_adc0:	ldi	XL,0x65
get_adc:	out	ADMUX,XL
		ldi	XL,0xc6
		out	ADCSR,XL
get_adc_1:	in	XL,ADCSR
		sbrc	XL,6
		rjmp	get_adc_1
		in	XL,ADCH
		ldi	XL,0xc6
		out	ADCSR,XL
get_adc_2:	in	XL,ADCSR
		sbrc	XL,6
		rjmp	get_adc_2
		in	XL,ADCH			;get data byte
		sbrc	XL,7
		ldi	XL,0x7f
		com	XL
		andi	XL,0x7f
		ldi	XH,42
		mul	XL,XH
		mov	XL,r1
		ret
.else
get_adc0:	ldi	XL,0x65
get_adc:	sts	ADMUX,XL
		ldi	XL,0xc6
		sts	ADCSRA,XL
get_adc_1:	lds	XL,ADCSRA
		sbrc	XL,6
		rjmp	get_adc_1
		lds	XL,ADCH
		ldi	XL,0xc6
		sts	ADCSRA,XL
get_adc_2:	lds	XL,ADCSRA
		sbrc	XL,6
		rjmp	get_adc_2
		lds	XL,ADCH
		sbrc	XL,7
		ldi	XL,0x7f
		com	XL
		andi	XL,0x7f
		ldi	XH,42
		mul	XL,XH
		mov	XL,r1
		ret
.endif


mess_0:		rcall	get_adc0
		mov	variable,XL
		set
		ret

mess_1:		rcall	get_adc1
		mov	variable,XL
		set
		ret

mess_2:		rcall	get_adc2
		mov	variable,XL
		set
		ret

mess_3:		rcall	get_adc3
		mov	variable,XL
		set
		ret

mess_4:		rcall	get_adc4
		mov	variable,XL
		set
		ret

mess_5:		rcall	get_adc5
		mov	variable,XL
		set
		ret


swi0:		rcall	get_adc0
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi0_1
		clt
		sbrs	r13,0
		set
swi0_1:		ret

swi1:		rcall	get_adc1
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi1_1
		clt
		sbrs	r13,0
		set
swi1_1:		ret


swi2:		rcall	get_adc2
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi2_1
		clt
		sbrs	r13,0
		set
swi2_1:		ret


swi3:		rcall	get_adc3
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi3_1
		clt
		sbrs	r13,0
		set
swi3_1:		ret


swi4:		rcall	get_adc4
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi4_1
		clt
		sbrs	r13,0
		set
swi4_1:		ret


swi5:		rcall	get_adc5
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	swi5_1
		clt
		sbrs	r13,0
		set
swi5_1:		ret


rswitch_a:	rcall	get_adc0
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_a_1
		clt
		sbrs	r13,0
		set
rswitch_a_1:	ret

rswitch_b:	rcall	get_adc1
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_b_1
		clt
		sbrs	r13,0
		set
rswitch_b_1:	ret

rswitch_c:	rcall	get_adc2
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_c_1
		clt
		sbrs	r13,0
		set
rswitch_c_1:	ret

rswitch_d:	rcall	get_adc3
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_d_1
		clt
		sbrs	r13,0
		set
rswitch_d_1:	ret

rswitch_e:	rcall	get_adc4
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_e_1
		clt
		sbrs	r13,0
		set
rswitch_e_1:	ret

rswitch_f:	rcall	get_adc5
		set
		sbrs	r13,0
		clt
		cpi	XL,5
		brcc	rswitch_f_1
		clt
		sbrs	r13,0
		set
rswitch_f_1:	ret

set_cdat:	set
		ret

send_cdat:	set
		ret

;-------------------------------------------------------------------------------
;switch variable display on
;-------------------------------------------------------------------------------
set_var:	mov	variable,r13
		cpi	variable,21
		brcs	set_var_1
		clr	variable
set_var_1:	set
		ret

mod_var:	add	variable,r13
		sbrc	variable,7
		clr	variable
		cpi	variable,21
		brcs	mod_var_1
		ldi	variable,20
mod_var_1:	set
		ret

view_var:	mov	r16,variable
		rcall	set_segm_num
		set
		ret

hide_var:	rcall	clr_segm
		set
		ret

var_gt:		set
		cp	r13,variable
		brcs	var_gt_1
		clt
var_gt_1:	ret

var_lt:		set
		cp	variable,r13
		brcs	var_lt_1
		clt
var_lt_1:	ret

var_eq:		set
		cp	variable,r13
		breq	var_eq_1
		clt
var_eq_1:	ret


var_ne:		set
		cp	variable,r13
		brne	var_eq_1
		clt
var_ne_1:	ret

inv_var:	ldi	XL,20
		sub	XL,variable
		mov	variable,XL
		set
		ret

rand_var:	mov	XL,r13
		inc	XL
		lds	XH,sys_rand2
		mul	XL,XH
		mov	variable,r1
		set
		ret

exchange1:	lds	XL,aux_var1
		sts	aux_var1,variable
		mov	variable,XL
		set
		ret

exchange2:	lds	XL,aux_var2
		sts	aux_var2,variable
		mov	variable,XL
		set
		ret

add1:		lds	XL,aux_var1
		add	variable,XL
		cpi	variable,21
		brcs	add1_1
		ldi	variable,20
add1_1:		set
		ret

add2:		lds	XL,aux_var2
		add	variable,XL
		cpi	variable,21
		brcs	add2_1
		ldi	variable,20
add2_1:		set
		ret



;-------------------------------------------------------------------------------
;switch segment display off
;-------------------------------------------------------------------------------
clr_segm:	clr	disp_d0
		clr	disp_d1
		ret

;-------------------------------------------------------------------------------
;set segment display to numeric value
;-------------------------------------------------------------------------------
set_segm_num:	push	ZL
		push	ZH
		push	XL
		push	XH
		mov	XL,r16	;E
		clr	XH	;Z
set_segm_num_1:	subi	XL,10
		brcs	set_segm_num_2
		inc	XH
		rjmp	set_segm_num_1
set_segm_num_2:	subi	XL,-10
		ldi	ZL,LOW(segtab_d*2)
		ldi	ZH,HIGH(segtab_d*2)
		add	ZL,XL
		adc	ZH,const_0
		lpm	XL,Z
		ldi	ZL,LOW(segtab_d*2)
		ldi	ZH,HIGH(segtab_d*2)
		add	ZL,XH
		adc	ZH,const_0
		lpm	XH,Z
		cpi	XH,0x3f
		brne	set_segm_num_3
		clr	XH
set_segm_num_3:	movw	disp_d0,XL
		pop	XH
		pop	XL
		pop	ZH
		pop	ZL
		ret

;-------------------------------------------------------------------------------
;set segment display to numeric value
;-------------------------------------------------------------------------------
set_segm_hex:	push	ZL
		push	ZH
		push	XL
		push	XH
		mov	XL,r16
		andi	XL,0x0f
		ldi	ZL,LOW(segtab_d*2)
		ldi	ZH,HIGH(segtab_d*2)
		add	ZL,XL
		adc	ZH,const_0
		lpm	XL,Z
		mov	XH,r16
		swap	XH
		andi	XH,0x0f
		ldi	ZL,LOW(segtab_d*2)
		ldi	ZH,HIGH(segtab_d*2)
		add	ZL,XH
		adc	ZH,const_0
		lpm	XH,Z
		movw	disp_d0,XL
		pop	XH
		pop	XL
		pop	ZH
		pop	ZL
		ret


;-------------------------------------------------------------------------------
;set segment display to numeric value
;-------------------------------------------------------------------------------
set_segm_num0:	push	ZL
		push	ZH
		push	XL
		mov	XL,r16			;value
		andi	XL,0x0f			;limit
		ldi	ZL,LOW(segtab_d*2)
		ldi	ZH,HIGH(segtab_d*2)
		add	ZL,XL
		adc	ZH,const_0
		lpm	disp_d0,Z
		pop	XL
		pop	ZH
		pop	ZL
		ret

;-------------------------------------------------------------------------------
;set pwm values (r16=value)
;-------------------------------------------------------------------------------
set_pwm:	push	ZL
		push	ZH
		cpi	r16,21
		brcs	set_pwm_1
		clr	r16
set_pwm_1:	ldi	ZL,LOW(pwmtab*2)
		ldi	ZH,HIGH(pwmtab*2)
		add	ZL,r16
		adc	ZH,const_0
		lpm	r0,Z
		pop	ZH
		pop	ZL
		ret


set_pwm0:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm,r0
		set
		ret

var_pwm0:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm,r0
		set
		ret

set_pwm1:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm+1,r0
		set
		ret

var_pwm1:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm+1,r0
		set
		ret

set_pwm2:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm+2,r0
		set
		ret

var_pwm2:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm+2,r0
		set
		ret

set_pwm3:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm+3,r0
		set
		ret

var_pwm3:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm+3,r0
		set
		ret
		
set_pwm4:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm+4,r0
		set
		ret

var_pwm4:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm+4,r0
		set
		ret

set_pwm5:	mov	r16,r13
		rcall	set_pwm
		sts	channel_pwm+5,r0
		set
		ret

var_pwm5:	mov	r16,variable
		rcall	set_pwm
		sts	channel_pwm+5,r0
		set
		ret

set_motor1:	ldi	XL,50
		mul	XL,r13
		lds	XL,cal_mot1l
		lds	XH,cal_mot1h
		add	XL,r0
		adc	XH,r1
sm1a:		get_tcnt1
		cpi	r16,64
		brne	sm1a
sm1b:		get_tcnt1
		cpi	r16,65
		brne	sm1b
		set_ocr1b
		set
		ret

var_motor1:	ldi	XL,50
		mul	XL,variable
		lds	XL,cal_mot1l
		lds	XH,cal_mot1h
		add	XL,r0
		adc	XH,r1
vm1a:		get_tcnt1
		cpi	r16,64
		brne	vm1a
vm1b:		get_tcnt1
		cpi	r16,65
		brne	vm1b
		set_ocr1b
		set
		ret

set_motor2:	ldi	XL,50
		mul	XL,r13
		lds	XL,cal_mot2l
		lds	XH,cal_mot2h
		add	XL,r0
		adc	XH,r1
sm2a:		get_tcnt1
		cpi	r16,64
		brne	sm2a
sm2b:		get_tcnt1
		cpi	r16,65
		brne	sm2b
		set_ocr1a
		set
		ret

var_motor2:	ldi	XL,50
		mul	XL,variable
		lds	XL,cal_mot2l
		lds	XH,cal_mot2h
		add	XL,r0
		adc	XH,r1
vm2a:		get_tcnt1
		cpi	r16,64
		brne	vm2a
vm2b:		get_tcnt1
		cpi	r16,65
		brne	vm2b
		set_ocr1a
		set
		ret



;-------------------------------------------------------------------------------
;segment table
;
;
;-------------------------------------------------------------------------------
;			   0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
segtab_d:	.db	0x3f,0x03,0x6d,0x67,0x53,0x76,0x7e,0x23,0x7f,0x77,0x7b,0x5e,0x3c,0x4f,0x7c,0x78

pwmtab:		.db	0,2,4,5,6,7,9,12,15,19,24,31,39,49,62,78,100,122,160,200,255,255

