//################################################################################
//#                                                                              #
//# (c)2012-2013 Joerg Wolfram   joerg@jcwolfram.de                              #
//#                                                                              #
//# Dieses Programm ist freie Software. Sie können es unter den Bedingungen      #
//# der GNU General Public License, wie von der Free Software Foundation         #
//# veröffentlicht, weitergeben und/oder modifizieren, entweder gemäß            #
//# Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.        #
//#                                                                              #
//# Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es       #
//# Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, auch ohne die     #
//# implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN          #
//# BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.      #
//#                                                                              #
//# Sie sollten ein Exemplar der GNU General Public License zusammen mit         #
//# diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free       #
//# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110,    #
//# USA.                                                                         #
//#                                                                              #
//# Jede Nutzung der Software/Informationen nonkonform zur GPL oder außerhalb    #
//# des Geltungsbereiches der GPL ist untersagt!                                 #
//#                                                                              #
//################################################################################
#include "main.h"

#include "avr_proto8.c"
#include "avr_proto88.c"

//------------------------------------------------------------------------------
// ausgelesenes Hexfile in Speicher einlesen
//------------------------------------------------------------------------------
int read_ihex()
{
	FILE *datei;
	char line[200];
	int bytes,index,rtype;
	unsigned int addr,ioffset;
	unsigned int rbytes;

	sprintf(line,"%s/lblocks/lblocks_read.hex",homedir);
	datei = fopen (line, "r");
	rbytes=-1;
	ioffset=0;

	if (datei != NULL)
	{
		rbytes=0;
		while((fscanf(datei,"%s\n",&line[0])) != EOF )
		{
			if(!(strncmp(line,":", 1)))
			{
				sscanf(strndup(&line[1], 2),"%x",&bytes);
				sscanf(strndup(&line[3], 4),"%x",&addr);
				sscanf(strndup(&line[7], 2),"%x",&rtype);
				if(rtype==0)
				{
					addr+=ioffset;
					for(index=0;index<bytes;index++)
					{
						sscanf(strndup(line+9+2*index, 2),"%x",&memdat[addr]);
						rbytes++;
						addr++;
					}
				}
			}
		}
		fclose (datei);
	}
	return rbytes;
}


//------------------------------------------------------------------------------
// Hexfile schreiben
//------------------------------------------------------------------------------
int write_ihex(void)
{
	FILE *datei;
	char line[200];
	int csum,i,j;
	unsigned int addr;

	sprintf(line,"%s/lblocks/lblocks.hex",homedir);
	datei = fopen (line, "w+");
	addr=0;

	if (datei != NULL)
	{
		for(i=0;i<256;i++)
		{
			csum=0x20;		//byte count
			csum+=(addr & 0xff);
			csum+=((addr >> 8) & 0xff);
			fprintf(datei,":20%04X00",addr);
			for(j=0;j<32;j++)
			{
				fprintf(datei,"%02X",memdat[addr+j]);
				csum+=memdat[addr+j];
			}
			csum=(255-(csum & 0xff)+1) & 0xff;
			fprintf(datei,"%02X\n",csum);
			addr+=32;
		}
		fprintf(datei,":00000001FF\n");
		fclose (datei);
		return 0;
	}
	else
	{
		return 1;
	}
}

//------------------------------------------------------------------------------
// Programm-Template zusammenstellen
//------------------------------------------------------------------------------
void copy_template()
{
	int addr;

	for(addr=0;addr<510;addr++)
	{
		if(mcu_type == 88)
		{
			memdat[addr]=avr_proto88[addr];
		}
		else
		{
			memdat[addr]=avr_proto8[addr];
		}
	}
	if(memdat[512] == 255)
	{
		for(addr=512;addr<1536;addr++)
		{
			if(mcu_type == 88)
			{
				memdat[addr]=avr_proto88[addr];
			}
			else
			{
				memdat[addr]=avr_proto8[addr];
			}
		}
	}
	if(memdat[1536] == 255)
	{
		for(addr=512;addr<1536;addr++)
		{
			if(mcu_type == 88)
			{
				memdat[addr]=avr_proto88[addr];
			}
			else
			{
				memdat[addr]=avr_proto8[addr];
			}
		}
	}
	if(memdat[2560] == 255)
	{
		for(addr=1536;addr<2560;addr++)
		{
			if(mcu_type == 88)
			{
				memdat[addr]=avr_proto88[addr];
			}
			else
			{
				memdat[addr]=avr_proto8[addr];
			}
		}
	}
	if(memdat[3584] == 255)
	{
		for(addr=3584;addr<4608;addr++)
		{
			if(mcu_type == 88)
			{
				memdat[addr]=avr_proto88[addr];
			}
			else
			{
				memdat[addr]=avr_proto8[addr];
			}
		}
	}
	for(addr=4608;addr<7680;addr++)
	{
		if(mcu_type == 88)
		{
			memdat[addr]=avr_proto88[addr];
		}
		else
		{
			memdat[addr]=avr_proto8[addr];
		}
	}
	//copy segtable only if empty
	if(memdat[7680] == 255)
	{
		for(addr=7680;addr<7936;addr++)
		{
			if(mcu_type == 88)
			{
				memdat[addr]=avr_proto88[addr];
			}
			else
			{
				memdat[addr]=avr_proto8[addr];
			}
		}
	}
	for(addr=7936;addr<8192;addr++)
	{
		if(mcu_type == 88)
		{
			memdat[addr]=avr_proto88[addr];
		}
		else
		{
			memdat[addr]=avr_proto8[addr];
		}
	}
}

//------------------------------------------------------------------------------
// Programm-Datenblock schreiben
//------------------------------------------------------------------------------
void create_data(int program)
{
	int addr,start_addr;
	int i,j,slen,box,ioset;
	int element,hnext,vnext,route;
	div_t coord1,coord2;

	start_addr = 512 + 1024 * program;
	slen=strlen(project_name);
	ioset = 0xff;
	if(slen > 32) slen = 32;
	for(i=0;i<slen;i++)
	{
		memdat[i+start_addr] = project_name[i];
	}
	if(slen < 32)
	{
		for(i=slen;i<32;i++)
		{
			memdat[i+start_addr] = 0;
		}
	}
	start_addr += 32;
	for(i=0;i<960;i++)
	{
		memdat[i+start_addr] = 255;	//dummy Vorbelegung
	}
	for(box=0;box<5;box++)
	{
		for(element = 0;element < 32; element++)
		{
			if(elements[box][element][0] < 100)
			{
				addr=start_addr+192*box+6*element;
				memdat[addr]=elements[box][element][1];		//type
				memdat[addr+1]=elements[box][element][2];	//value
				memdat[addr+4]=elements[box][element][0];	//position
				coord1 = div(elements[box][element][0],12);
				hnext = 255;
				vnext = 255;
				route = 255;
				//find direct hor
				for(i=0;i<32;i++)
				{
					coord2 = div(elements[box][i][0],12);
					if(((coord1.rem + 1) == coord2.rem) && (coord1.quot == coord2.quot)) 
					{
						hnext = i;
					}
					if(((coord1.quot + 1) == coord2.quot) && (coord1.rem == coord2.rem)) 
					{
						vnext = i;
					}
				
					for(j=0;j<32;j++)
					{
						if((routes[box][j][0] == element) && (routes[box][j][1] == i))
						{
							hnext = i;
							route = j;
						}
					}
				}
				memdat[addr+2] = hnext;
				memdat[addr+3] = vnext;
				memdat[addr+5] = route;
				if(elements[box][element][1] == 70) ioset &= 0xdf;
				if(elements[box][element][1] == 71) ioset &= 0xef;
				if(elements[box][element][1] == 72) ioset &= 0xf7;
				if(elements[box][element][1] == 73) ioset &= 0xfb;
				if(elements[box][element][1] == 74) ioset &= 0xfd;
				if(elements[box][element][1] == 75) ioset &= 0xfe;
				if(elements[box][element][1] == 86) ioset &= 0xdf;
				if(elements[box][element][1] == 87) ioset &= 0xef;
				if(elements[box][element][1] == 88) ioset &= 0xf7;
				if(elements[box][element][1] == 89) ioset &= 0xfb;
			}
		}
	}
	memdat[start_addr+960] = ioset;	//io config
}

//------------------------------------------------------------------------------
// Programm-Datenblock schreiben
//------------------------------------------------------------------------------
void readout_data(int program)
{
	int addr,start_addr;
	int i,box;
	int element,route,element1,element2;
	start_addr = 512 + 1024 * program;

	//copy name
	for(i=0;i<32;i++)
	{
		project_name[i]=memdat[i+start_addr];
	}
	project_name[32]=0;

	start_addr+=32;

	//init field
	for(box=0;box<5;box++)
	{
		for(i=0;i<32;i++)
		{
			elements[box][i][0]=255;
			elements[box][i][1]=255;
			elements[box][i][2]=0;
			routes[box][i][0]=255;
			routes[box][i][1]=255;
		}
	}
	//read elements
	for(box=0;box<5;box++)
	{
		for(element=0;element < 32; element++)
		{
			addr=start_addr+192*box+6*element;
			if(memdat[addr] < 100)
			{
				elements[box][element][1] = memdat[addr];		//type
				elements[box][element][2] = memdat[addr+1];		//value
				elements[box][element][0] = memdat[addr+4];		//position
			}
		}
	}
	//read routes
	for(box=0;box<5;box++)
	{
		for(element = 0;element < 32; element++)
		{
			addr=start_addr+192*box+6*element;
			if(memdat[addr+5] < 100)
			{
				element1 = element;
				element2 = memdat[addr+2];
				route = memdat[addr+5];
				routes[box][route][0] = element1;
				routes[box][route][1] = element2;
			}
		}
	}
}

//------------------------------------------------------------------------------
// read memory data from device
//------------------------------------------------------------------------------
int download(void)
{
	char cmd[500];
	int fd,i,addr;
	unsigned char tbuffer;


	//now try the programmer
	if(mcu_type == 8)
	{
		sprintf(cmd,"%s/lblocks/scripts/readscript_m8.sh",homedir);
	}
	else
	{
		sprintf(cmd,"%s/lblocks/scripts/readscript_m88.sh",homedir);
	}
	if(system(cmd) != 0) 
	{
		copy_template();
		return 1;
	}
	if(read_ihex() < 0) 
	{
		copy_template();
		return 2;
	}
	copy_template();
	return 0;
}


//------------------------------------------------------------------------------
// write memory data to device
//------------------------------------------------------------------------------
int upload(void)
{
	char cmd[500];
	int fd,i,addr;
	unsigned char tbuffer;

	//programmer
	if(write_ihex() > 0) return 3;
	if(mcu_type == 8)
	{
		sprintf(cmd,"%s/lblocks/scripts/progscript_m8.sh",homedir);
	}
	else
	{
		sprintf(cmd,"%s/lblocks/scripts/progscript_m88.sh",homedir);
	}
	if(system(cmd) != 0) return 4;
	return 0;
}


